xquery version "3.1";

(:~
 : This module contains some helper functions that are relevant for creating the
 : HTML version of the motifs.
 :)

module namespace tei2html-motifs="http://ahikar.sub.uni-goettingen.de/ns/tei2html/motifs";

declare namespace tei="http://www.tei-c.org/ns/1.0";



declare function tei2html-motifs:get-corresponding-end-of-motif($start-of-motif as processing-instruction(),
    $node as element(tei:ab))
as processing-instruction() {
    let $mid := substring-after($start-of-motif/string(), 'mid="') => substring-before('"')
    let $motif-ends := $node//processing-instruction("oxy_comment_end")
    return
        if ($mid) then
            $motif-ends[matches(., concat('mid="', $mid))]
        else
            $motif-ends[not(contains(., "mid"))]
};

declare function tei2html-motifs:get-unclosed-motifs-before-line($node as element(tei:ab))
as processing-instruction("oxy_comment_start")* {
    let $preceding-motif-starts := $node/preceding::processing-instruction("oxy_comment_start")
    return
        $preceding-motif-starts[not(following::node()[self::processing-instruction("oxy_comment_end")][. >>  $preceding-motif-starts and . << $node])]
};


declare function tei2html-motifs:make-data-next-attribute($motif-start as processing-instruction(),
    $node as element(tei:ab))
as attribute() {
    attribute data-next {
        let $next-line := $node/following-sibling::tei:ab[1]
        return
            generate-id($next-line) || ".motif." || tei2html-motifs:get-motif-name($motif-start)
    }
};

declare function tei2html-motifs:get-motif-name($motif-start as processing-instruction())
as xs:string {
    substring-after($motif-start/string(), 'comment="') => substring-before('"')
};

declare function tei2html-motifs:make-attributes-for-mark($motif-start as processing-instruction(),
    $node as element(tei:ab))
as attribute()+ {
    let $motif-name := tei2html-motifs:get-motif-name($motif-start)
    return
        (
            attribute data-id {generate-id($node) || ".motif." || $motif-name},
            attribute class {"motif"},
            attribute data-type {$motif-name}
        )
};

declare function tei2html-motifs:is-part-of-one-or-more-unclosed-motifs($node as element(tei:ab))
as xs:boolean {
    count(tei2html-motifs:get-unclosed-motifs-before-line($node)) ge 1
};

declare function tei2html-motifs:is-part-of-more-than-one-unclosed-motif($node as element(tei:ab))
as xs:boolean {
    count(tei2html-motifs:get-unclosed-motifs-before-line($node)) gt 1
};

declare function tei2html-motifs:get-corresponding-motif-start($motif-end as processing-instruction(),
    $node as element(tei:ab))
as processing-instruction()? {
    let $mid :=
        if (matches($motif-end, "mid")) then
            substring-after($motif-end/string(), 'mid="') => substring-before('"')
        else
            ()
    let $previous-unclosed-motif-starts := tei2html-motifs:get-unclosed-motifs-before-line($node)
    return
        if ($mid) then
            $previous-unclosed-motif-starts[matches(., concat('mid="', $mid))][1]
        else
            $previous-unclosed-motif-starts[not(matches(., 'mid="'))][1]
};

declare function tei2html-motifs:is-line-in-motif($line as element(tei:ab))
as xs:boolean {
    let $preceding-comment-start := $line/preceding::processing-instruction("oxy_comment_start")[1]
    let $preceding-comment-end := $line/preceding::processing-instruction("oxy_comment_end")[1]
    return
        if ($preceding-comment-start
        and 
            ($preceding-comment-start >> $preceding-comment-end
            or not($preceding-comment-end))
        ) then
            true()
        else
            false()
};

declare function tei2html-motifs:get-nodes-before-pi($pi as processing-instruction(),
    $node as element(tei:ab))
as node()* {
    $node/child::node()[. << $pi]
};
