xquery version "3.1";

(: 
 : This module contains the unit tests for the motifs processed in tei2html.xqm. 
 :)

module namespace t="http://ahikar.sub.uni-goettingen.de/ns/motifs/tests";

declare namespace tei="http://www.tei-c.org/ns/1.0";

import module namespace tei2html="http://ahikar.sub.uni-goettingen.de/ns/tei2html" at "../modules/tei2html.xqm";
import module namespace test="http://exist-db.org/xquery/xqsuite" at "resource:org/exist/xquery/lib/xqsuite/xqsuite.xql";

(: case 1: motif takes up complete line :)
declare
    %test:assertTrue
function t:test-sample-data-motifs-case1() {
    let $sample-data :=
        <body xmlns="http://www.tei-c.org/ns/1.0" xml:lang="syc">
            <ab>
                <?oxy_comment_start comment="adherence_wisdom"?>
                Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
                <?oxy_comment_end?>
            </ab>
        </body>
    let $reference-output :=
        <xhtml:div xmlns:xhtml="http://www.w3.org/1999/xhtml" id="" class="body syc">
            <div id="" class="ab">
                <mark data-id="N1.2.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom">
                        Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
                        </mark>
            </div>
        </xhtml:div>
    
    return
        local:are-reference-and-output-identical($sample-data, $reference-output)
};

(: case 2: motif takes up parts of line :)
declare
    %test:assertTrue
function t:test-sample-data-motifs-case2() {
    let $sample-data :=
        <body xmlns="http://www.tei-c.org/ns/1.0" xml:lang="syc">
            <ab>
                Text davor <?oxy_comment_start comment="adherence_wisdom"?>
                Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
                <?oxy_comment_end?> text danach
            </ab>
        </body>
    let $reference-output :=
        <xhtml:div xmlns:xhtml="http://www.w3.org/1999/xhtml" id="" class="body syc">
            <div id="" class="ab">
                        Text davor <mark data-id="N1.2.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom">
                        Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
                        </mark> text danach
                </div>
        </xhtml:div>
    
    return
        local:are-reference-and-output-identical($sample-data, $reference-output)
};

(: case 3.1: motif takes up (parts of) two lines :)
declare
    %test:assertTrue
function t:test-sample-data-motifs-case3_1() {
    let $sample-data :=
    <body xmlns="http://www.tei-c.org/ns/1.0" xml:lang="syc">
        <!-- Fall 1: Kommentar spannt sich über ganze Zeile -->
        <ab>
                Text davor <?oxy_comment_start comment="adherence_wisdom"?>
                Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
        </ab>
        <ab>
            hier geht das Motiv noch weiter. <?oxy_comment_end?> Text nach dem Motiv.
        </ab>
    </body>
    let $reference-output :=
        <xhtml:div xmlns:xhtml="http://www.w3.org/1999/xhtml" id="" class="body syc">
            <div id="" class="ab">
                        Text davor <mark data-id="N1.3.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom" data-next="N1.4.motif.adherence_wisdom">
                        Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
                </mark>
            </div>
            <div id="" class="ab">
                <mark data-id="N1.4.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom">
                    hier geht das Motiv noch weiter. </mark> Text nach dem Motiv.
                </div>
        </xhtml:div>
    
    return
        local:are-reference-and-output-identical($sample-data, $reference-output)
};


(: case 3.2: motif takes up (parts of) multiple lines :)
declare
    %test:assertTrue
function t:test-sample-data-motifs-case3_2() {
    let $sample-data :=
        <body xmlns="http://www.tei-c.org/ns/1.0" xml:lang="syc">
            <ab>
                    Text davor <?oxy_comment_start comment="adherence_wisdom"?>
                    Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
            </ab>
            <ab>diese Line ist auch noch vollständig im Motiv.</ab>
            <ab>
                hier geht das Motiv noch weiter. <?oxy_comment_end?> Text nach dem Motiv.
            </ab>
        </body>
    let $reference-output :=
        <xhtml:div xmlns:xhtml="http://www.w3.org/1999/xhtml" id="" class="body syc">
            <div id="" class="ab">
                        Text davor <mark data-id="N1.2.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom" data-next="N1.3.motif.adherence_wisdom">
                        Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
                </mark>
            </div>
            <div id="" class="ab">
                <mark data-id="N1.3.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom" data-next="N1.4.motif.adherence_wisdom">
                diese Line ist auch noch vollständig im Motiv.</mark>
            </div>
            <div id="" class="ab">
                <mark data-id="N1.4.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom">
                    hier geht das Motiv noch weiter. </mark> Text nach dem Motiv.
                </div>
        </xhtml:div>
    
    return
        local:are-reference-and-output-identical($sample-data, $reference-output)
};


(: case 4: two nested motifs encompassing the whole line :)
declare
    %test:assertTrue
function t:test-sample-data-motifs-case4() {
    let $sample-data :=
        <body xmlns="http://www.tei-c.org/ns/1.0" xml:lang="syc">
            <ab>
                    <?oxy_comment_start comment="adherence_wisdom"?>
                    <?oxy_comment_start comment="loyalty" mid="2"?>
                    Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
                    <?oxy_comment_end mid="2"?>
                    <?oxy_comment_end?>
            </ab>
        </body>
    let $reference-output :=
        <xhtml:div xmlns:xhtml="http://www.w3.org/1999/xhtml" id="" class="body syc">
            <div id="" class="ab">
                <mark data-id="N1.2.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom">
                    <mark data-id="N1.2.motif.loyalty" class="motif" data-type="loyalty">
                        Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
                        </mark>
                </mark>
            </div>
        </xhtml:div>
    
    return
        local:are-reference-and-output-identical($sample-data, $reference-output)
};


(: case 5: two nested motifs encompassing parts of the line :)
declare
    %test:assertTrue
function t:test-sample-data-motifs-case5() {
    let $sample-data :=
        <body xmlns="http://www.tei-c.org/ns/1.0" xml:lang="syc">
            <ab>
                    Etwas Text vor dem ersten Motiv.
                    <?oxy_comment_start comment="adherence_wisdom"?>
                    Hier steht noch etwas Text dazwischen.
                    <?oxy_comment_start comment="loyalty" mid="2"?>
                    Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
                    <?oxy_comment_end mid="2"?>
                    Noch ein bisschen Text nach dem Motiv.
                    <?oxy_comment_end?>
                    .... und der Schluß.
            </ab>
        </body>
    let $reference-output :=
        <xhtml:div xmlns:xhtml="http://www.w3.org/1999/xhtml" id="" class="body syc">
            <div id="" class="ab">
                        Etwas Text vor dem ersten Motiv.
                        <mark data-id="N1.2.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom">
                        Hier steht noch etwas Text dazwischen.
                        <mark data-id="N1.2.motif.loyalty" class="motif" data-type="loyalty">
                        Daß alle unsere Erkenntnis mit der Erfahrung anfange, daran 
                        </mark>
                        Noch ein bisschen Text nach dem Motiv.
                        </mark>
                        .... und der Schluß.
                </div>
        </xhtml:div>
    
    return
        local:are-reference-and-output-identical($sample-data, $reference-output)
};

(: case 6: two nested motifs encompassing (parts of) several lines :)
declare
    %test:assertTrue
function t:test-sample-data-motifs-case6() {
    let $sample-data :=
        <body xmlns="http://www.tei-c.org/ns/1.0" xml:lang="syc">
            <ab>
                    Etwas Text vor dem ersten Motiv.
                    <?oxy_comment_start comment="adherence_wisdom"?>
                    Hier steht noch etwas Text dazwischen.
                    Daß alle unsere <?oxy_comment_start comment="loyalty" mid="2"?>Erkenntnis mit der Erfahrung anfange, daran 
            </ab>
            <ab>Hier ist noch Text dazwischen.</ab>
            <ab>Teesti test test test</ab>
            <ab>
                    Noch ein bisschen Text<?oxy_comment_end mid="2"?> nach dem Motiv.
                    <?oxy_comment_end?>
                    .... und der Schluß.
            </ab>
        </body>
    let $reference-output :=
        <xhtml:div xmlns:xhtml="http://www.w3.org/1999/xhtml" id="" class="body syc">
            <div id="" class="ab">
                Etwas Text vor dem ersten Motiv.
                <mark data-id="N1.2.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom" data-next="N1.3.motif.adherence_wisdom">
                    Hier steht noch etwas Text dazwischen.
                    Daß alle unsere <mark data-id="N1.2.motif.loyalty" class="motif" data-type="loyalty" data-next="N1.3.motif.loyalty">Erkenntnis mit der Erfahrung anfange, daran 
                    </mark>
                </mark>
            </div>
            <div id="" class="ab">
                <mark data-id="N1.3.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom" data-next="N1.4.motif.adherence_wisdom">
                    <mark data-id="N1.3.motif.loyalty" class="motif" data-type="loyalty" data-next="N1.4.motif.loyalty">Hier ist noch Text dazwischen.</mark>
                </mark>
            </div>
            <div id="" class="ab">
                <mark data-id="N1.4.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom" data-next="N1.5.motif.adherence_wisdom">
                    <mark data-id="N1.4.motif.loyalty" class="motif" data-type="loyalty" data-next="N1.5.motif.loyalty">Teesti test test test</mark>
                </mark>
            </div>
            <div id="" class="ab">
                <mark data-id="N1.5.motif.adherence_wisdom" class="motif" data-type="adherence_wisdom">
                    <mark data-id="N1.5.motif.loyalty" class="motif" data-type="loyalty">
                        Noch ein bisschen Text</mark> nach dem Motiv.
                </mark>
                .... und der Schluß.
            </div>
        </xhtml:div>
    
    return
        local:are-reference-and-output-identical($sample-data, $reference-output)
};


(: case 7: two nested motifs encompassing (parts of) several lines where the
 : motifs begin in different lines each :)
declare
    %test:assertTrue
function t:test-sample-data-motifs-case6() {
    let $sample-data :=
        <body xmlns="http://www.tei-c.org/ns/1.0" xml:lang="syc">
            <ab>Text vor dem ersten Motiv.
                <?oxy_comment_start comment="erstes_motiv"?>
                Beginn des ersten Motivs.
            </ab>
            <ab>
                Fortsetzung des ersten Motivs.
                <?oxy_comment_start comment="zweites_motiv" mid="2"?>
                Beginn des zweiten Motivs.
            </ab>
            <ab>
                Eine Zeile, die komplett in beiden Motiven liegt.
            </ab>
            <ab>
                Schluss des zweiten Motifs.
                <?oxy_comment_end mid="2"?>
                ... und der Schluss des ersten Motivs.
                <?oxy_comment_end?>
                Noch ein bisschen Text nach den beiden Motiven.
            </ab>
        </body>
    let $reference-output :=
        <xhtml:div xmlns:xhtml="http://www.w3.org/1999/xhtml" id="" class="body syc">
            <div id="" class="ab">Text vor dem ersten Motiv. <mark data-id="N1.2.motif.erstes_motiv"
                    class="motif" data-type="erstes_motiv" data-next="N1.3.motif.erstes_motiv"> Beginn des
                    ersten Motivs. </mark>
            </div>
            <div id="" class="ab">
                <mark data-id="N1.3.motif.erstes_motiv" class="motif" data-type="erstes_motiv"
                    data-next="N1.4.motif.erstes_motiv"> Fortsetzung des ersten Motivs. <mark
                        data-id="N1.3.motif.zweites_motiv" class="motif" data-type="zweites_motiv"
                        data-next="N1.4.motif.zweites_motiv"> Beginn des zweiten Motivs. </mark>
                </mark>
            </div>
            <div id="" class="ab">
                <mark data-id="N1.4.motif.erstes_motiv" class="motif" data-type="erstes_motiv"
                    data-next="N1.5.motif.erstes_motiv">
                    <mark data-id="N1.4.motif.zweites_motiv" class="motif" data-type="zweites_motiv"
                        data-next="N1.5.motif.zweites_motiv"> Eine Zeile, die komplett in beiden Motiven
                        liegt. </mark>
                </mark>
            </div>
            <div id="" class="ab">
                <mark data-id="N1.5.motif.erstes_motiv" class="motif" data-type="erstes_motiv">
                    <mark data-id="N1.5.motif.zweites_motiv" class="motif" data-type="zweites_motiv">
                        Schluss des zweiten Motifs. </mark> ... und der Schluss des ersten Motivs. </mark>
                Noch ein bisschen Text nach den beiden Motiven. </div>
        </xhtml:div>
    
    return
        local:are-reference-and-output-identical($sample-data, $reference-output)
};


declare function local:are-reference-and-output-identical($sample-data as node(),
    $reference-output as node())
as xs:boolean {
    let $transformed-sample-data := local:prepare-data(tei2html:transform($sample-data))
    let $reference-data := local:prepare-data($reference-output)
    return
        deep-equal($transformed-sample-data, $reference-data)
};

declare function local:prepare-data($data as node())
as xs:string{
    serialize($data)
    => replace("[\n\t\r]", "")
    => replace("[\s]+", "")
};
